###############################################################################
## XBar 0.9 pre 4
## code from Kroller - v0.93b
## stefanux @ libero .it
## http://www.xiaprojects.com
###############################################################################
##
##
###############################################################################
##  Kroller - v0.93b
###############################################################################
##
##  Brandon Zehm <caspian at dotconf.net>
##
##  Kroller is an extension for SuperKaramba to make a MacOS-X like
##  menu bar.
##  It was originally based on tuxbar, but now only a few lines of
##  code from it remain.
##
##  This SuperKaramba extension was designed to be flexible.
##  You can add new icons, or change existing ones by simply
##  editing the lines in the "User Configurable Section" in this
##  file.  You can use icons of any size you want.  All you need
##  to do is make sure that all the "small" icons are the same
##  size, and all the "big" icons are the same size, and you set
##  those sizes in the imageSize and imageSizeBig variables.
##
##  Feel free to email me with comments or suggestions!
##
###############################################################################

## This import statement allows access to the karamba functions
import karamba

## Import a few standard python modules
import string, re, os


## osX Bar extensions
import thread
import XBarGetMail
import XBarSensor
import XBarEngine


##import xcursor
try:
	import xcursor
except ImportError:
        global widgetCheckForMouseMax
        print "XCursor lib failed to load, auto reset with animations will not work!"
        class MyCursor:
                def position(self):
                        return 0,0
        xcursor=MyCursor()
        widgetCheckForMouseMax=0


## The time module causes problems on some systems (i.e. Redhat 9)
## so we are a little cautious loading it.  If it fails we redefine
## the time function with our own implementation.
try:
    import time
except ImportError:
    print "WARNING: The time module failed to load.  Switching to fallback implementation.  Some time related functions (like maxFPS) may not function properly!"
    class MyTime:
        def time(self):
            os.times()[-1]

        def sleep(self, time=0):
            return

    time = MyTime()



############################
##  Global Configuration  ##
############################
conf = {

    ## Configuration file location, DO NOT NORMALLY EDIT THIS!  Recent enhancements
    ## in SuperKaramba allow us to locate the conf file on our own.  If you
    ## do hard-code a path here it will still work, but you shouldn't need to.
    "configurationFile"      : "kroller.conf",

    "version"                : "0.9",

    ## The following variables are overwritten by the configuration file
    "animateOnClick"         : "none",                    ## Animate icons when clicked
    "detectResolution"       : 0,                         ## Auto detection of resolution
    "resolutionX"            : 0,                         ## Manually specified X resolution
    "resolutionY"            : 0,                         ## Manually specified Y resolution
    "autoPosition"           : 0,                         ## Automatically position kroller or not
    "maxFPS"                 : 0,                         ## Maximum widget updates / second.  0 == no limit.
    "smallImageSize"         : 0,                         ## Size of small images
    "largeImageSize"         : 0,                         ## Size of large images
    "iconSeparation"         : "default",                 ## Icon Separation
    "currentIconEffect"      : "none",                    ## Effect for current icon
    "useBackgroundBar"       : 0,                         ## Enable/disable background bar image
    "barImages"              : "barimages/",              ## Directory where to find bar images
    "backgroundBarColor"     : "",                        ## Color or background bar.. this need to be a directory under the barImages dir!
    "autoHide"               : 0,                         ## Enable/disable auto-hiding
    "autoHideDelay"          : 0,                         ## Seconds of idle time to activate autoHide
    "virtualDesktopSelector" : 0,                         ## Enable/disable virtual desktop selector
    "numVirtualDesktops"     : 0,                         ## Number of virtual desktops to display
    "virtualDesktopIconX"    : 0,                         ## Pixels wide the virtual desktop image is
    "virtualDesktopIconY"    : 0,                         ## Pixels tall the virtual desktop image is
    "debug"                  : 0,                         ## Debug level
    "overTexty"              : 0,
    "overTextHeight"         : 0,
    "overTextSize"           : 0,
    "overTextSpace"          : 0,
    "TaskSleepMax"           : 0,
    "TaskMax"                : 0,
    "TaskAddAutoCenter"      : 0,
    "BounceAddedIcon"        : 0,
    "BounceOldIcon"          : 0,
    "ThePlayCmd"             : 0,
    "MenuAddExtraMenu"       : 0,
}


## Define some more global variables
## The values are not set here!
widgetCheckForMouseMax=1
widgetCheckForMouse = 0
TheLastIconSelected = -1

## Icon related vars
commands         = []
imageFiles       = []
images           = []
imagesBig        = []
imageSize        = 0
imageSizeBig     = 0
numIcons         = 0
currentIconEffect= "none"
animateOnClick   = "none"
kdeIconPaths     = []

## Over Text Patch
## TheText
overText = 0
overTextStrings = []
## Y Displacenment
overTexty = 15
## The Max height of the Text
overTextHeight = 20
overTextSize = 16
overTextSpace =5

## Arrows Task Patch
## the list of Arrows Black Images
arrows = []
## Window title to match
commandsAlt = []

## System Informations
## Emails
SysEmailIndex=0
SysEmailInterval = 0
SysEmailFileYes = 0
SysEmailFileNO = 0

OtherIconsCmd = []
OtherIconsIcon = []
OtherIconsText =[]

## Battery
## Email
## Moved into Sensors


## Array of pop3 servers
SysEMailServers=[]
SysEMailUsers=[]
SysEMailPasswords=[]

## Add Icons Task Patch
## The number of CONFIGURATION Icons
numIconsCfg = 0
## The actual Counter displayed
TaskSleep = []
TaskDelta = 60
TaskSleepMax=5
TaskMax=5
TaskListIgnore = []
TaskSeparator = 0
## These are for the left click
SpecialCommand = []
SpecialAction = []

## Window list
PopUpWindowList = 0
SelectWindowID = []
SelectWindowTask = []


## Mini task actions
WinActionCloseFile="mini_close.png"
WinActionMaxFile="mini_close.png"
WinActionMinFile="mini_close.png"
WinActionCloseDY=0
WinActionCloseDX=0

mouseLastX = 0
mouseLastY = 0


## Icon placement variables
iconSeparation   = 0
centerSmallIcon  = 0
iconFloor        = 0
raiseIcons       = 0
xZoomBoundaryMin = 0
xZoomBoundaryMax = 0
yZoomBoundaryMin = 0
yZoomBoundaryMax = 0


## Widget parameters
borderLeft       = 0
borderRight      = 0
borderBottom     = 0
widgetWidth      = 0
widgetHeight     = 0
resolutionX      = 0
resolutionY      = 0

## Icon resizing related parameters
sizeDiff         = 0
sizeRatio        = 0

## Auto Hiding
autoHide         = 0
autoHideDelay    = 0

## Time related vars
sleepTime        = 0
lastActiveTime   = 0

## Variables for background bar support
useBackBar       = 0
backBarLeft      = 0
backBarCenter    = 0
backBarRight     = 0
backBarBorder    = 4

backBarVDCenter  = 0
backBarVDRight   = 0

## These are used for storing current icon placement info
currentXPosition = 0
currentYPosition = 0
currentSize      = 0

## Virtual Desktop Related
virtualDesktop   = []
virtualDesktopActive   = []
virtualDesktopRefresh=0
ActiveDesktop=1
GetVirtualDesktopCmd="dcop kwin KWinInterface currentDesktop"
xVdBoundaryMin   = 0
xVdBoundaryMax   = 0
yVdBoundaryMin   = 0
yVdBoundaryMax   = 0

## Misc
isReset          = 0    ## To improve CPU efficiency, we use this to keep track of the state of the widget
debug            = 0    ## Debugging level

GlobalStatus = 0

## Thread bug fix for superkaramba
FakeActionsIndex =[]
FakeActionsImages =[]
FakeActionsIndexT=[]
FakeActionsText=[]



def XBarCreateDesktopSelector(widget):
	global conf
	global virtualDesktop
	global virtualDesktopActive
	global widgetHeight
	global borderBottom
	global borderLeft
	global iconSeparation
	global numIcons
	global xVdBoundaryMin
	global xVdBoundaryMax
	global yVdBoundaryMin
	global yVdBoundaryMax
	curicon = "top"
	initialDesktopNumTop = 1
	initialDesktopNumBottom = (conf["numVirtualDesktops"]/2) + 1  ## FIXME this will only work if there are an even number of virtual desktops being displayed
	## Setup the background bar for the virtual desktop area
	if (useBackBar >= 1):
		print "Please, crate a background for the DesktopSelector :) [stefanux@libero.it]"
		##backBarVDCenter =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
		##backBarVDRight  =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-right.png"))
		##karamba.resizeImage(widget, backBarVDCenter, (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2)) + iconSeparation, imageSize + (backBarBorder * 2))
		##karamba.resizeImageSmooth(widget, backBarVDRight, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
		##karamba.moveImage(widget, backBarVDCenter, borderLeft + (iconSeparation*(numIcons)), iconFloor - backBarBorder)
		##karamba.moveImage(widget, backBarVDRight, borderLeft + (iconSeparation*(numIcons+1)) + (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2)), iconFloor - backBarBorder)
	## Create enough empty entries in virtualDesktop
	for i in range(0, conf["numVirtualDesktops"] + 1, 1):
		virtualDesktop.append(0)
		virtualDesktopActive.append(0)
	## For each virtual desktop (VD) place an icon on the screen and assign a command to run when clicked
	for i in range(0, conf["numVirtualDesktops"], 1):
		x = borderLeft + (iconSeparation*(numIcons+1)) + (int(i/2)*conf["virtualDesktopIconX"])
		y = widgetHeight - borderBottom - (conf["virtualDesktopIconY"]*2)
		if (curicon == "top"):
			desktopNum = initialDesktopNumTop
			initialDesktopNumTop = initialDesktopNumTop + 1
			curicon = "bottom"
		else:
			y = y + conf["virtualDesktopIconY"]
			desktopNum = initialDesktopNumBottom
			initialDesktopNumBottom = initialDesktopNumBottom + 1
			curicon = "top"
		virtualDesktopActive[desktopNum] = (karamba.createImage(widget, 0, 0, conf["barImages"] + "virtual_desktop_active.png"))
		virtualDesktop[desktopNum] = (karamba.createImage(widget, 0, 0, conf["barImages"] + "virtual_desktop.png"))
		karamba.resizeImage(widget, virtualDesktop[desktopNum],   conf["virtualDesktopIconX"], conf["virtualDesktopIconY"])
		karamba.moveImage(widget, virtualDesktop[desktopNum], x, y)
		karamba.resizeImage(widget, virtualDesktopActive[desktopNum],   conf["virtualDesktopIconX"], conf["virtualDesktopIconY"])
		karamba.moveImage(widget, virtualDesktopActive[desktopNum], x, y)
	## These vars are used in widgetMouseMoved() but pre-calculated here once for CPU savings
	## Width the Task patch we have to calcolate every time we add or remove a icon
	xVdBoundaryMin = borderLeft + (iconSeparation*(numIcons+1))
	xVdBoundaryMax = borderLeft + (iconSeparation*(numIcons+1)) + ((conf["numVirtualDesktops"]/2) * conf["virtualDesktopIconX"])
	yVdBoundaryMin = widgetHeight - borderBottom - (conf["virtualDesktopIconY"]*2) - conf["virtualDesktopIconY"]
	yVdBoundaryMax = widgetHeight - borderBottom

def XBarUpdateDesktopSelector(widget):
	global conf
	global GetVirtualDesktopCmd
	global conf
	global virtualDesktop
	global widgetHeight
	global borderBottom
	global borderLeft
	global iconSeparation
	global numIcons
	global xVdBoundaryMin
	global xVdBoundaryMax
	global yVdBoundaryMin
	global yVdBoundaryMax
	global ActiveDesktop
	global sleepTime
	if (sleepTime):
		time.sleep(sleepTime)
	if (conf["virtualDesktopSelector"] == 1):
		## Setup some variables
		ActiveDesktop=os.popen(GetVirtualDesktopCmd,"r")
		ActiveDesktop=int(string.strip(ActiveDesktop.read(10)))
		for i in range(1, conf["numVirtualDesktops"]+1, 1):
			if (sleepTime):
				time.sleep(sleepTime*10)
			if(i==ActiveDesktop):
				karamba.hideImage(widget,virtualDesktop[i])
				karamba.showImage(widget,virtualDesktopActive[i])
			else:
				karamba.showImage(widget,virtualDesktop[i])
				karamba.hideImage(widget,virtualDesktopActive[i])



def menuOptionChanged(widget, key, value):
	print "User modified the configuration: " + str(key) + " "+str(value)
	if(key=="debug"):
		debug=value
	if(key=="currentIconEffect"):
		if(value==0):
			currentIconEffect="none"
		else:
			currentIconEffect="highlight"
	if(key=="autoHide"):
		autoHide=value
	if(key=="useBackBar"):
		useBackBar=value
		if(useBackBar==0):
			karamba.hideImage(widget,backBarLeft)
			karamba.hideImage(widget,backBarRight)
			karamba.hideImage(widget,backBarCenter)
		else:
			karamba.showImage(widget,backBarLeft)
			karamba.showImage(widget,backBarRight)
			karamba.showImage(widget,backBarCenter)
	if(key=="virtualDesktopSelector"):
		virtualDesktopSelector=value
		if(virtualDesktopSelector==0):
			for i in range(0, conf["numVirtualDesktops"], 1):
				karamba.hideImage(widget,virtualDesktop[i])
		else:
			for i in range(0, conf["numVirtualDesktops"], 1):
				karamba.showImage(widget,virtualDesktop[i])





def XBarCreateMenuConfiguration(widget):
	global debug
	global currentIconEffect
	global autoHide
	global conf
	global useBackBar
	global autoHideDelay
	karamba.addMenuConfigOption(widget,"debug","Wanna debug?")
	karamba.setMenuConfigOption(widget,"debug", str(debug))
	karamba.addMenuConfigOption(widget,"currentIconEffect","Disable OverIconEffect")
	if(currentIconEffect=="none"):
		karamba.setMenuConfigOption(widget,"currentIconEffect", str(0))
	else:
		karamba.setMenuConfigOption(widget,"currentIconEffect", str(1))
	karamba.addMenuConfigOption(widget,"autoHide",str("Auto Hide after ") + str(autoHideDelay))
	karamba.setMenuConfigOption(widget,"autoHide", str(autoHide))
	karamba.addMenuConfigOption(widget,"virtualDesktopSelector","Show Desktop Selector")
	karamba.setMenuConfigOption(widget,"virtualDesktopSelector", str(conf["virtualDesktopSelector"]))
	karamba.addMenuConfigOption(widget,"useBackBar","Show Background bar")
	karamba.setMenuConfigOption(widget,"useBackBar", str(useBackBar))


def XBarUpdateText(widget,theIndex,newText):
	global overTextStrings
	overTextStrings[theIndex]=newText


def XBarFakePostUpdateIcon(widget):
	global FakeActionsIndex
	global FakeActionsImages
	localFakeActionsIndex=[]
	localFakeActionsImages=[]
	## We hope the thread don't write here?
	## We copy data to be fastest
	for i in range(len(FakeActionsIndex)):
		localFakeActionsIndex.append(FakeActionsIndex[i])
		localFakeActionsImages.append(FakeActionsImages[i])
	FakeActionsIndex=[]
	FakeActionsImages=[]
	## Now with all time required we update the icons
	for i in range(len(localFakeActionsIndex)):
		XBarUpdateIcon(widget,localFakeActionsIndex[i],localFakeActionsImages[i])


def XBarFakeUpdateIcon(widget,theIndex,newFile):
	global FakeActionsIndex
	global FakeActionsImages
	## This function will update the icons on next cicle
	FakeActionsIndex.append(theIndex)
	FakeActionsImages.append(newFile)


def XBarFakePostUpdateText(widget):
	global FakeActionsIndexT
	global FakeActionsText
	localFakeActionsIndex=[]
	localFakeActionsImages=[]
	## We hope the thread don't write here?
	## We copy data to be fastest
	for i in range(len(FakeActionsIndexT)):
		localFakeActionsIndex.append(FakeActionsIndexT[i])
		localFakeActionsImages.append(FakeActionsText[i])
	FakeActionsIndexT=[]
	FakeActionsText=[]
	## Now with all time required we update the icons
	for i in range(len(localFakeActionsIndex)):
		XBarUpdateText(widget,localFakeActionsIndex[i],localFakeActionsImages[i])


def XBarFakeUpdateText(widget,theIndex,newFile):
	global FakeActionsIndexT
	global FakeActionsText
	FakeActionsIndexT.append(theIndex)
	FakeActionsText.append(newFile)

## XBarUpdateIcon
## This will change an icon of the bar
## Graphics lib osX Bar
def XBarUpdateIcon(widget,theIndex,newFile):
	global images
	global imagesBig
	global borderLeft
	global centerSmallIcon
	global iconSeparation
	global iconFloor
	global imageFiles
	global widgetHeight
	global widgetWidth
	global resolutionX
	global resolutionY
	global imageSize
	global mouseLastX
	global mouseLastY
	global isReset
	Oldimage=images[theIndex]
	OldimageBig=imagesBig[theIndex]
	if(os.path.exists(newFile)):
		imageFiles[theIndex]=newFile
	else:
		imageFiles[theIndex]=findIcon(widget, newFile, str(imageSizeBig) + "x" + str(imageSizeBig))
	if(imageFiles[theIndex]!=-1):
		images[theIndex]=karamba.createImage(widget, borderLeft + centerSmallIcon + (theIndex*iconSeparation), iconFloor, imageFiles[theIndex])
		imagesBig[theIndex]=(karamba.createImage(widget, borderLeft + (theIndex*iconSeparation), widgetHeight, imageFiles[theIndex]))
		karamba.resizeImageSmooth(widget, images[theIndex], imageSize, imageSize)
		karamba.deleteImage(widget,Oldimage)
		karamba.deleteImage(widget,OldimageBig)
		if(isReset == 0):
			## i think you are moving :)
			karamba.moveImage(widget, images[theIndex], ((theIndex)*iconSeparation)+imageSizeBig, widgetHeight)
			##x, y = xcursor.position()
			widgetMouseMoved(widget, mouseLastX, mouseLastY, 0)



## XBarAnimateReset
## Animations when we reset the Bar
## Graphics lib osX Bar
def XBarAnimateReset(widget):
	global currentXPosition
	global currentYPosition
	global currentSize
	global lastActiveTime
	global TheLastIconSelected
	global sleepTime
	global imageSizeBig
	global imageSize
	global raiseIcons
	global iconFloor
	x, y = xcursor.position()
	if(y<(resolutionY-widgetHeight-50)):
		moltiplier=float(360)/(raiseIcons*3)
		sizediff=float(currentSize-imageSize)/(iconFloor-currentYPosition)
		if(TheLastIconSelected>=0 and TheLastIconSelected<numIcons):
			for down in range (1,iconFloor-currentYPosition,4):
				resetanimatsize=sizediff*(iconFloor-currentYPosition-down)+imageSize
				karamba.moveImage(widget, imagesBig[TheLastIconSelected], (currentXPosition+currentSize/2)-resetanimatsize/2, currentYPosition + down)
				##karamba.rotateImage(widget, imagesBig[TheLastIconSelected], int(down * moltiplier))

				karamba.resizeImage(widget, imagesBig[TheLastIconSelected], resetanimatsize, resetanimatsize)
				karamba.redrawWidget(widget)
				if (sleepTime):
					time.sleep(sleepTime)
					time.sleep(sleepTime)
				## Sleep a bit to accomadate the maxFPS setting
			for down in range (1,raiseIcons*2,4):
				karamba.moveImage(widget, imagesBig[TheLastIconSelected],  (currentXPosition+currentSize/2)-imageSize/2, iconFloor -down)
				##karamba.rotateImage(widget, imagesBig[TheLastIconSelected], int(down * moltiplier))
				karamba.redrawWidget(widget)
				if (sleepTime):
					time.sleep(sleepTime)
					time.sleep(sleepTime)
			for down in range (1,raiseIcons,4):
				karamba.moveImage(widget, imagesBig[TheLastIconSelected],  (currentXPosition+currentSize/2)-imageSize/2, iconFloor -raiseIcons +down)
				##karamba.rotateImage(widget, imagesBig[TheLastIconSelected], int(down * moltiplier))
				karamba.redrawWidget(widget)
				if (sleepTime):
					time.sleep(sleepTime)
					time.sleep(sleepTime)
			TheLastIconSelected=-1;
			if (sleepTime):
				time.sleep(sleepTime)

## XBarSysInit
## Initializations of Sys Engine
## Graphics lib osX Bar
def XBarSysInit(widget):
	## Setup for Engine
	##XBarEngine.EngineSetup(widget,XBarUpdateIcon,XBarUpdateText)
	XBarEngine.EngineSetup(widget,XBarFakeUpdateIcon,XBarFakeUpdateText,XBarAddSensorIcons)
	## Init for Sensors
	XBarSensor.XBarSensorInit(widget,XBarEngine)
	## Setup for Sensors
	XBarSensor.XBarSensorLoad(XBarEngine.SysSensorListIndex,XBarEngine.SysSensorListType,XBarEngine.SysSensorListCmd)
	## Free up memory
	## I hope to move this part to the Engine
	##SysSensorListIndex=[]
	##SysSensorListType=[]
	##SysSensorListCmd=[]



def menuItemClicked(widget,Ilmenu,command):
	global SelectWindowTask
	global SelectWindowID
	print "Menu command"
	if(Ilmenu==PopUpWindowList):
		for idcerca in range (len(SelectWindowTask)):
			if(command==SelectWindowID[idcerca]):
				if(type(SelectWindowTask[idcerca])==str):
					os.system(SelectWindowTask[idcerca] + " & ")
				else:
					karamba.performTaskAction(widget,SelectWindowTask[idcerca],5)
					print "Window activated"



def XBarClickOverTaskIcon(widget,theIcon,MouseButton,x,y):
	## got the task
	global SpecialCommand
	global commands
	global PopUpWindowList
	global SelectWindowID
	global SelectWindowTask

	SpecialActionCommand=0
	for cerca in range(len(SpecialCommand)):
		if(SpecialCommand[cerca]==commands[theIcon]):
			SpecialActionCommand=SpecialAction[cerca]
	if(SpecialActionCommand==0 or SpecialActionCommand==3 ):
		LocalTaskList=karamba.getTaskList(widget)
		TaskFounded=-1
		if(MouseButton!=1):
			## Middle Click
			if(PopUpWindowList!=0):
				karamba.deleteMenu(widget,PopUpWindowList)
			PopUpWindowList=karamba.createMenu(widget);
			SelectWindowID=[]
			SelectWindowTask=[]
			if(conf["MenuAddExtraMenu"]>0):
				SelectWindowID.append(karamba.addMenuItem(widget,PopUpWindowList,str("New: ") + overTextStrings[theIcon],imageFiles[theIcon]))
				SelectWindowID.append(karamba.addMenuItem(widget,PopUpWindowList,str("Close: ") +overTextStrings[theIcon],conf["barImages"] + "mini_close.png"))
				print conf["barImages"] + "mini_close.png"
				SelectWindowTask.append(commands[theIcon])
		for task in LocalTaskList:
			## command
			TaskName = karamba.getTaskInfo(widget, task)[2]
			## Title
			TaskProc = karamba.getTaskInfo(widget, task)[0]
			if (TaskProc == commandsAlt[theIcon] or string.find(TaskProc,commandsAlt[theIcon])!=-1):
				## got the title
				TaskFounded=task
		if(TaskFounded==-1 or MouseButton!=1):
			for task in LocalTaskList:
				## command
				TaskName = karamba.getTaskInfo(widget, task)[2]
				## Title
				TaskProc = karamba.getTaskInfo(widget, task)[0]
				if (TaskName == commands[theIcon] or TaskName == commandsAlt[theIcon] or string.find(TaskName,commandsAlt[theIcon])!=-1):
					## got the title
					TaskFounded=task
					SelectWindowID.append(karamba.addMenuItem(widget,PopUpWindowList,TaskProc,imageFiles[theIcon]))
					SelectWindowTask.append(task)
				##karamba.performTaskAction(widget,TaskFounded,5)
		if(TaskFounded!=-1):
			if(MouseButton!=1 and len(SelectWindowTask)>1):
				karamba.popupMenu(widget,PopUpWindowList,x,y)
			else:
				if(SpecialActionCommand==3):
					print "Close: " + commands[theIcon]
					karamba.performTaskAction(widget,TaskFounded,4)
				if(SpecialActionCommand==0):
					## from version 0.8.2 if we click on a task in foreground, it will be minimized
					if(karamba.getTaskInfo(widget, TaskFounded)[7]==1):
						print "Window minimized"
						## 3
						karamba.performTaskAction(widget,TaskFounded,8)
					else:
						## 8
						## 5
						karamba.performTaskAction(widget,TaskFounded,8)
						print "Window activated"
		else:
			print "Running: " + commands[theIcon]
			if(conf['ThePlayCmd']!=0):
				## we can play audio!
				os.system(conf['ThePlayCmd'] + " sounds/boing.wav & ")
			os.system(commands[theIcon] + " & ")
	if(SpecialActionCommand==1):
		print "Running: " + commands[theIcon]
		os.system(commands[theIcon] + " & ")
	if(SpecialActionCommand==2):
		print "Noting to do"



def XBarRelocate(widget):
	global widgetWidth
	global widgetHeight
	global xVdBoundaryMin
	global xVdBoundaryMax
	global borderleft
	global numIcons
	global iconSeparation
	global conf
	global virtualDesktop
	global backBarVDCenter
	global backBarVDRight
	global backBarBorder
	if (conf["virtualDesktopSelector"] == 1):
		initialDesktopNumTop = 1
		initialDesktopNumBottom = (conf["numVirtualDesktops"]/2) + 1
		curicon="top"
		for i in range(0, conf["numVirtualDesktops"], 1):
			x = borderLeft + (iconSeparation*(numIcons+1)) + (int(i/2)*conf["virtualDesktopIconX"])
			y = widgetHeight - borderBottom - (conf["virtualDesktopIconY"]*2)
			if (curicon == "top"):
				desktopNum = initialDesktopNumTop
				initialDesktopNumTop = initialDesktopNumTop + 1
				curicon="bottom"
			else:
				y = y + conf["virtualDesktopIconY"]
				desktopNum = initialDesktopNumBottom
				initialDesktopNumBottom = initialDesktopNumBottom + 1
				curicon="top"
			karamba.moveImage(widget, virtualDesktop[desktopNum], x, y)
			karamba.moveImage(widget, virtualDesktopActive[desktopNum], x, y)
			## karamba.createClickArea(widget, x, y, conf["virtualDesktopIconX"], 	conf["virtualDesktopIconY"], "dcop kwin KWinInterface setCurrentDesktop " + str(desktopNum))
		xVdBoundaryMin = borderLeft + (iconSeparation*(numIcons+1))
		xVdBoundaryMax = borderLeft + (iconSeparation*(numIcons+1)) + ((conf["numVirtualDesktops"]/2) * conf["virtualDesktopIconX"])
		widgetWidth = (borderLeft + (iconSeparation * numIcons) + borderRight)
	else:
		## enlarge
		widgetWidth = (borderLeft + (iconSeparation * numIcons) + borderRight)
	## redraw widget
	karamba.resizeWidget(widget, widgetWidth, widgetHeight)
	if(conf["TaskAddAutoCenter"]==1):
		## We have to move the OsX Docker
		if (int(conf['autoPosition']) == 1):
			karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-widgetHeight) )
	resetWidget(widget)

def XBarAnimateBounce(widget,where,iconIndex):
	global iconFloor
	global iconSeparation
	global centerSmallIcon
	global borderLeft
	global raiseIcons
	global sleepTime

	if(where==0):
		## Bounce effect on added icon
		a=(iconIndex*iconSeparation) + borderLeft + centerSmallIcon
		b=iconFloor
		for up in range (1,raiseIcons,4):
			karamba.moveImage(widget, images[iconIndex], a, b - up)
			karamba.redrawWidget(widget)
			## Sleep a bit to accomadate the maxFPS setting
			if (sleepTime):
				time.sleep(sleepTime)
				time.sleep(sleepTime)
		for down in range (1,raiseIcons*2,4):
			karamba.moveImage(widget, images[iconIndex], a,b+down-raiseIcons)
			karamba.redrawWidget(widget)
			## Sleep a bit to accomadate the maxFPS setting
			if (sleepTime):
				time.sleep(sleepTime)
				time.sleep(sleepTime)
		## Going up
		for up in range (1,raiseIcons,4):
			karamba.moveImage(widget, images[iconIndex], a, b+ raiseIcons - up)
			karamba.redrawWidget(widget)
			## Sleep a bit to accomadate the maxFPS setting
			if (sleepTime):
				time.sleep(sleepTime)
				time.sleep(sleepTime)


def XBarAddSensorIcons(widget,Testo,Icona,sCmd):
	global OtherIconsCmd
	global OtherIconsIcon
	global OtherIconsText
	founded=0
	for i in range(len(OtherIconsText)):
		founded=0
		if(OtherIconsText[i]==Testo):
			if(i==0):
				## We are at the First icon
				## new cicle, new list :)
				OtherIconsText=[]
				OtherIconsIcon=[]
				OtherIconsCmd=[]
			else:
				founded=1
			break
	if(founded==0):
		OtherIconsCmd.append(sCmd)
		OtherIconsIcon.append(Icona)
		OtherIconsText.append(Testo)

def XBarAddOverTastIconsPost(widget,NewTaskName,NewWindowName,NewCommand):
##########################################
##  This Function Add a left Icon       ##
##########################################
	global numIcons
	global widgetWidth
	global imageFiles
	global imagesBig
	global images
	global arrows
	global TaskDelta
	global imageSize
	global imageSizeBig
	global commands
	global commandsAlt
	global commands
	global commands
	global imageFiles
	global TaskSleep
	global numIcons
	global iconSeparation
	global borderLeft
	global xZoomBoundaryMax
	global arrows
	global overTextStrings
	global SpecialCommand
	global SpecialAction
	global TaskListIgnore
	global sleepTime

	## We are searching for the Right icons
	TheFoundedIcon=str(NewTaskName)
	## 32x32
	TheFoundedIcon=findIcon(widget, TheFoundedIcon, str(imageSizeBig) + "x" + str(imageSizeBig))
	## if we have found the icon
	if (TheFoundedIcon == -1):
		## TheFoundedIcon=str(NewWindowName) + ".png"
		TheFoundedIcon=str(NewWindowName)
		## 32x32
		TheFoundedIcon=findIcon(widget, TheFoundedIcon, str(imageSizeBig) + "x" + str(imageSizeBig))
	if (TheFoundedIcon != -1):
		## Adding the Icon
		numIcons=numIcons+1
		## the 32 icon
		imageFiles.append(TheFoundedIcon)
		## default Command
		commands.append(NewCommand)
		## Remove Timer set to 0
		TaskSleep.append(0)
		## Window Title
		commandsAlt.append(NewWindowName)
		overTextStrings.append(NewWindowName)
		## Index
		i=numIcons-1
		## Add small image
		images.append(karamba.createImage(widget, borderLeft + centerSmallIcon + (i*iconSeparation), iconFloor, imageFiles[i]))
		imagesBig.append(karamba.createImage(widget, borderLeft + (i*iconSeparation), widgetHeight, imageFiles[i]))
		## Add Arrows
		arrows.append(karamba.createImage(widget, borderLeft + (i*iconSeparation)+imageSize/2, iconFloor+imageSize, "arrow.png"))
		karamba.hideImage(widget,arrows[i])
		## Add clickable area that runs a command
		## Done by the new engine by the 0.4 version
		##for cerca in range(len(SpecialCommand)):
		##	if(SpecialCommand[cerca]==commands[i]):
		##		if(SpecialAction[cerca]==1):
	        ##			karamba.createClickArea(widget, borderLeft + (i*iconSeparation), (widgetHeight - borderBottom - imageSizeBig - 	raiseIcons), iconSeparation, imageSizeBig + (raiseIcons/2), commands[i])
		## Resize the image just in case the image wasn't imageSize pixels wide
		karamba.resizeImageSmooth(widget, images[i], imageSize, imageSize)
		## Recalculate the Mouse Move
		## xZoomBoundaryMin = (borderLeft-iconSeparation)
		xZoomBoundaryMax = (borderLeft+(iconSeparation*(numIcons+1)))
		## yZoomBoundaryMin = 0
		## yZoomBoundaryMax = widgetHeight
		XBarRelocate(widget)
		## superkaramba 0.26 bug!
		##karamba.changeImageIntensity(widget, images[i], 0.7, 300)
		for conta in range(conf['BounceAddedIcon']):
			XBarAnimateBounce(widget,0,i)
	else:
		## Icon not found add it to black list :)
		TaskListIgnore.append(NewTaskName)

def XBarAddOverTaskIcons(widget,NewTaskName,NewWindowName):
	XBarAddOverTastIconsPost(widget,NewTaskName,NewWindowName,NewTaskName)




def XBarListSensorCheck(widget):
	global OtherIconsIcon
	global OtherIconsCmd
	global OtherIconsText
	for i in range(len(OtherIconsText)):
		founded=0
		for j in range(numIcons):
			if(commandsAlt[j]==OtherIconsText[i]):
				founded=1
				## Reset Timer
				TaskSleep[j]=0
				break
		if(founded==0):
			## we have to add the icon:
			XBarAddOverTastIconsPost(widget,OtherIconsIcon[i],OtherIconsText[i],OtherIconsCmd[i])





## TIME Based Arrow and so on REFRESH
def widgetTaskUpdate(widget):
	## VAR
	global arrows
	global TaskList
	global commands
	global images
	global imagesBig
	global imageFiles
	global commandsAlt
	global numIcons
	global numIconsCfg
	global TaskSleepMax
	global isReset
	global TaskMax
	global TaskListIgnore
	global sleepTime
	## We dont' have found any task
	TaskFound=0
	## List of Window showed
	TaskList = karamba.getTaskList(widget)
	## Removing the Arrows, this will flicker? i hope no!
	for i in range(numIcons):
		## Arrows
		karamba.hideImage(widget, arrows[i])
		## increse the sleep time before remove the task icons
		TaskSleep[i]=TaskSleep[i]+1
	## Selecting ony of all process
	for task in TaskList:
		## getting task informations
		TaskName = karamba.getTaskInfo(widget, task)[2]
		TaskProc = karamba.getTaskInfo(widget, task)[0]
		TaskFound=0
		for i in range(numIcons):
			## He have found the right icon, show the arrow
			if TaskName == commands[i] or TaskName == commandsAlt[i] or TaskProc == commands[i] or TaskProc == commandsAlt[i] or string.find(TaskProc,commandsAlt[i])!=-1:
				karamba.showImage(widget, arrows[i])
				## Task Found!
				TaskFound=1
				if(TaskSleep[i]>1 and isReset==1):
					## May be the program is launched 2second ago
					## superkaramba 0.26 bug
					## karamba.changeImageIntensity(widget,images[i],2,300)
					for conta in range(conf['BounceOldIcon']):
						XBarAnimateBounce(widget,0,i)
						if (sleepTime):
							time.sleep(sleepTime)
							time.sleep(sleepTime)
				TaskSleep[i]=0
		# Task not in list
		if(TaskFound==0):
			## Check for the Maximun number of icons
			if(TaskMax>(numIcons-numIconsCfg)):
				escluso=0
				for cerca in range(len(TaskListIgnore)):
					if(TaskListIgnore[cerca]==TaskProc):
						escluso=1
					if(TaskListIgnore[cerca]==TaskName):
						escluso=1
				if(escluso==0):
					## We have to add the Icon
					XBarAddOverTaskIcons(widget,TaskName,TaskProc)
				else:
					print "Task Escluso"
	## Removing the Added Tasks
	for i in range(numIcons):
		## After 15 seconds?
		if(i<numIcons):
			if(TaskSleep[i]>TaskSleepMax):
				## Reset Counter
				TaskSleep[i]=0
				## if it's a added icon, we remove it
				if(i>=numIconsCfg):
					if(i!=numIconsCfg-1):
						## The Last!
						isReset=0
					XBarRemoveOverTaskIcon(widget,i)


def XBarRemoveOverTaskIcon(widget,IconIndex):
	global commands
	global commandsAlt
	global commands
	global commands
	global imageFiles
	global TaskSleep
	global numIcons
	global iconSeparation
	global borderLeft
	global xZoomBoundaryMax
	global arrows
	global overTextStrings
	global sleepTime

	if(conf['ThePlayCmd']!=0):
		## we can play audio!
		os.system(conf['ThePlayCmd'] + " sounds/close.wav & ")
	print "Removing: " +str(IconIndex)
	## We have to remove the tasks added icons
	commands.pop(IconIndex)
	commandsAlt.pop(IconIndex)
	imageFiles.pop(IconIndex)
	TaskSleep.pop(IconIndex)
	overTextStrings.pop(IconIndex)
	## Animation sliding out
	a=(IconIndex*iconSeparation) + borderLeft + centerSmallIcon
	b=iconFloor
	for slideout in range (1,raiseIcons,4):
		karamba.moveImage(widget, images[IconIndex], a,  b-slideout)
		karamba.redrawWidget(widget)
		## Sleep a bit to accomadate the maxFPS setting
		if (sleepTime):
			time.sleep(sleepTime)
			time.sleep(sleepTime)
	for slideout in range (1,raiseIcons*3,4):
		karamba.moveImage(widget, images[IconIndex], a, slideout+ raiseIcons +b)
		karamba.redrawWidget(widget)
		## Sleep a bit to accomadate the maxFPS setting
		if (sleepTime):
			time.sleep(sleepTime)
			time.sleep(sleepTime)


	karamba.hideImage(widget,images.pop(IconIndex))
	karamba.hideImage(widget,imagesBig.pop(IconIndex))
	karamba.hideImage(widget,arrows.pop(IconIndex))

	numIcons=numIcons-1
	## Recalculate the Mouse Move
	## xZoomBoundaryMin = (borderLeft-iconSeparation)
	xZoomBoundaryMax = (borderLeft+(iconSeparation*(numIcons+1)))
	## yZoomBoundaryMin = 0
	## yZoomBoundaryMax = widgetHeight
	XBarRelocate(widget)
	karamba.redrawWidget(widget)

def XBarSearchConfig(widget):
	###################################
	##  Read the configuration file  ##
	###################################
	global conf
	## Try hard to find the configuration file, otherwise exit with an error message
	themePath = karamba.getThemePath(widget)
	if (os.path.exists(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])):
		print "Loading configuration from home directory: ", os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"]
		readConfigurationFile(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])
	elif (os.path.exists(karamba.getThemePath(widget) + "/" + conf["configurationFile"])):
		print "Loading configuration from theme directory: ", karamba.getThemePath(widget) + "/" + conf["configurationFile"]
		readConfigurationFile(karamba.getThemePath(widget) + "/" + conf["configurationFile"])
	elif (os.path.exists(conf["configurationFile"])):
		print "Loading configuration from manually specified location: ", conf["configurationFile"]
		readConfigurationFile(conf["configurationFile"])
	else:
		print
		print "ERROR - Loading the configuration file failed!  The configuration file could not be located!"
		print "Hint: try putting kroller.conf in ~/.superkaramba/"



###############################################################################
##  Function:
##    initWidget(widget)
##
##  Description:
##    This function is buggy.. I think it gets called too early, because
##    if you try to create an image it crashes karamba.
###############################################################################
def initWidget(widget):
    """Initialize Widget"""

    global conf
    global debug
    global images
    global imagesBig
    global commands
    global imageFiles
    global images
    global imagesBig
    global imageSize
    global imageSizeBig
    global numIcons
    global currentIconEffect
    global animateOnClick
    global iconSeparation
    global centerSmallIcon
    global borderLeft
    global borderRight
    global borderBottom
    global sizeDiff
    global sizeRatio
    global autoHide
    global autoHideDelay
    global iconFloor
    global raiseIcons
    global xZoomBoundaryMin
    global xZoomBoundaryMax
    global yZoomBoundaryMin
    global yZoomBoundaryMax
    global widgetHeight
    global widgetWidth
    global sleepTime
    global lastActiveTime
    global useBackBar
    global backBarLeft
    global backBarCenter
    global backBarRight
    global resolutionX
    global resolutionY
    global virtualDesktop
    global xVdBoundaryMin
    global xVdBoundaryMax
    global yVdBoundaryMin
    global yVdBoundaryMax
    global overText
    global numIconsCfg
    global overTextStrings
    global TaskSeparator
    global widgetCheckForMouseMax

    global overTexty
    global overTextHeight
    global overTextSpace
    global overTextSize
    global TaskSleepMax
    global TaskMax

    global backBarVDCenter
    global backBarVDRight

    global GlobalStatus

    if(GlobalStatus==0):
        ## We are at 1
        GlobalStatus=GlobalStatus+1
        XBarSearchConfig(widget)
    ## Sleep time based on maxFPS from configuration file
    if (conf['maxFPS'] != 0):
        sleepTime = (1.0 / conf['maxFPS'])
    if (sleepTime):
        time.sleep(sleepTime)

    ## Make sure numeric configuration parameters are set that way
    imageSize    = int(conf['smallImageSize'])
    imageSizeBig = int(conf['largeImageSize'])
    debug = int(conf['debug'])
    currentIconEffect = conf['currentIconEffect']
    autoHide = int(conf['autoHide'])
    autoHideDelay = int(conf['autoHideDelay'])
    animateOnClick = conf['animateOnClick']
    conf['BounceAddedIcon']=int(conf['BounceAddedIcon'])
    conf["virtualDesktopSelector"] = int(conf["virtualDesktopSelector"])
    conf["virtualDesktopIconX"] = int(conf["virtualDesktopIconX"])
    conf["virtualDesktopIconY"] = int(conf["virtualDesktopIconY"])
    useBackBar = int(conf['useBackgroundBar'])

    ## Patch conf
    overTexty = int(conf["overTexty"])
    overTextHeight = int(conf["overTextHeight"])
    overTextSize = int(conf["overTextSize"])
    overTextSpace = int(conf["overTextSpace"])
    TaskSleepMax= int(conf["TaskSleepMax"])
    TaskMax=int(conf["TaskMax"])
    ## System informations
    conf["TaskAddAutoCenter"]=int(conf["TaskAddAutoCenter"])
    if(widgetCheckForMouseMax>0):
        widgetCheckForMouseMax=int(conf["OnMouseOutReset"])
    if (sleepTime):
        time.sleep(sleepTime)
    ## After readed the configuration we create the User Configuration menu
    XBarCreateMenuConfiguration(widget)
    ## If numVirtualDesktops is "auto" find out the current value from the users
    ## KDE config files.
    if (conf["numVirtualDesktops"] == "auto"):
        ## Try and find a good kdeglobals file
        if (os.path.exists(os.getenv("HOME") + "/.kde/share/config/kdeglobals")):
            conf["numVirtualDesktops"] = getKDEConfigValue(os.getenv("HOME") + "/.kde/share/config/kdeglobals", "Desktops", "Number")
        elif (os.path.exists(os.getenv("KDEDIR") + "/share/config/kdeglobals")):
            conf["numVirtualDesktops"] = getKDEConfigValue(os.getenv("KDEDIR") + "/share/config/kdeglobals", "Desktops", "Number")

        if ( (conf["numVirtualDesktops"] == "auto") or (conf["numVirtualDesktops"] == -1) ):
            print
            print "ERROR - Could not automatically determin the number of virtual desktops in use!"
            print "Hint: Try manually specifying the \"numVirtualDesktops\" setting in kroller.conf"
            print
            conf["virtualDesktopSelector"] = 0
            conf["numVirtualDesktops"] = 0
            ## FIXME: sys.exit(1)

    ## Make sure its an int
    conf["numVirtualDesktops"] = int(conf["numVirtualDesktops"])
    if (sleepTime):
        time.sleep(sleepTime)


    ## <calculations>

    ## Calculate some other important stuff
    if (re.search("default", str(conf['iconSeparation']))):
        iconSeparation = (imageSize + imageSizeBig)/2            ## iconSeparation is the size of the "cells" where icons go
        if (iconSeparation > (2*imageSize)):
            iconSeparation = (2*imageSize)
    else:
        if (re.search("\D", str(conf['iconSeparation']))):
            print "ERROR - The iconSeparation parameter in the configuration file is not \"default\" or a numeric value!"
        else:
            iconSeparation = int(conf['iconSeparation'])
            iconSeparation = iconSeparation + imageSize

    if (sleepTime):
        time.sleep(sleepTime)
    ######################################
    ##  Setup System Informations       ##
    ######################################
    XBarSysInit(widget)
    if (sleepTime):
        time.sleep(sleepTime)

    numIcons = len(imageFiles)                               ## Self explanitory

    ## Patch for Added Icons
    numIconsCfg=numIcons


    centerSmallIcon = ((iconSeparation/2)-(imageSize/2))     ## The number of pixels to center each small icon in it's "cell"
    borderLeft = iconSeparation                              ## How much "white" space on the left
    borderRight = iconSeparation                             ## How much "white" space on the left
    if (conf["virtualDesktopSelector"] == 1):                ## If virtual desktop selector is enabled make room for it
        conf["virtualDesktopIconY"] = imageSize/2
        conf["virtualDesktopIconX"] = int(conf["virtualDesktopIconY"] * (4.0/3))  ## Use a 4/3 aspect ratio for virtual desktop icons
        borderRight += (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2))
        if (useBackBar >= 1):
            borderRight += imageSize
    borderBottom = 4                                        ## How much "white" space on the bottom
    ## Patch borderBottom = 10                                        ## How much "white" space on the bottom
    sizeDiff = imageSizeBig - imageSize                      ## How many pixels are there from the small icon to the big icon
    sizeRatio = ((sizeDiff + 0.0) / imageSizeBig)            ## Ratio of small icon to big icon
    raiseIcons = 10                                          ## How far to "raise" icons up when icon is at its largest size
    ## Patch raiseIcons = 30                                          ## How far to "raise" icons up when icon is at its largest size

    ## Calculate widget size
    widgetWidth = (borderLeft + (iconSeparation * numIcons) + borderRight)
    widgetHeight = imageSizeBig + raiseIcons + borderBottom + overTextHeight

    ## Figure out where the bottom of the icons should be (the iconFloor)
    iconFloor = (widgetHeight - imageSize - borderBottom)

    ## Boundaries in which we do icon zooming
    ## These will probably have to become more complex formulas in the future
    xZoomBoundaryMin = (borderLeft-iconSeparation)
    xZoomBoundaryMax = (borderLeft+(iconSeparation*(numIcons+1)))
    yZoomBoundaryMin = 0
    yZoomBoundaryMax = widgetHeight

    if (sleepTime):
        time.sleep(sleepTime)
    if (sleepTime):
        time.sleep(sleepTime)
    ## If we're in debugging mode display the current kroller environment
    if (debug >= 1):
        print "numIcons: ", numIcons
        print "imageSize: ", imageSize
        print "imageSizeBig:", imageSizeBig
        print "centerSmallIcon: ", centerSmallIcon
        print "iconSeparation: ", iconSeparation
        print "iconFloor: ", iconFloor
        print "sizeDiff: ", sizeDiff
        print "sizeRatio: ", sizeRatio
        print "borderLeft: ", borderLeft
        print "borderRight: ", borderRight
        print "borderBottom: ", borderBottom
        print "raiseIcons: ", raiseIcons
        print "widgetWidth: ", widgetWidth
        print "widgetHeight: ", widgetHeight
        print "sleepTime: ", sleepTime

    ## </calculations>

    ###################################
    ##  Detect current X resolution  ##
    ###################################
    if (int(conf['detectResolution']) == 1):
        detectResolution()
    else:
        resolutionX = int(conf['resolutionX'])
        resolutionY = int(conf['resolutionY'])
        print "Using specified resolution: " + str(resolutionX) + "x" + str(resolutionY)

    ## Set the size of our widget since we're just starting
    karamba.resizeWidget(widget, widgetWidth, widgetHeight)


    ################################
    ##  Setup the background bar  ##
    ################################
    if (sleepTime):
        time.sleep(sleepTime)

    if (useBackBar == 1):
        ## Create image references
        backBarLeft =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-left.png"))
        backBarCenter = (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
        backBarRight =  (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-right.png"))
        ## Set initial sizes
        karamba.resizeImageSmooth(widget, backBarLeft, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
        karamba.resizeImageSmooth(widget, backBarRight, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        ## Set initial positions
        karamba.moveImage(widget, backBarLeft, borderLeft - backBarBorder , iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)
    if (useBackBar == 2):
        ## Create image references
        backBarLeft =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-left.png"))
        backBarCenter = (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
        backBarRight =  (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-right.png"))
        ## Set initial sizes
        karamba.resizeImageSmooth(widget, backBarLeft, backBarBorder , imageSize + (backBarBorder * 2))
        karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
        karamba.resizeImageSmooth(widget, backBarRight,  backBarBorder , imageSize + (backBarBorder * 2))
        ## Set initial positions
        karamba.moveImage(widget, backBarLeft, borderLeft - backBarBorder , iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)

    TaskSeparator = (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-separator.png"))
    karamba.hideImage(widget,TaskSeparator)
    ## karamba.resizeImageSmooth(widget, TaskSeparator, iconSeparation/4, imageSize + (backBarBorder * 2))
    karamba.moveImage(widget, TaskSeparator, borderLeft + (numIconsCfg * iconSeparation)-iconSeparation/8, iconFloor)

    if (sleepTime):
        time.sleep(sleepTime)
    ######################################
    ##  Setup Virtual Desktop Selector  ##
    ######################################
    XBarCreateDesktopSelector(widget)

    if (sleepTime):
        time.sleep(sleepTime)
    ##########################################
    ##  Setup All the normal rolling icons  ##
    ##########################################
    for i in range (0,numIcons):
        if (sleepTime):
            time.sleep(sleepTime)
        ## First do some work to find the icon the user wants.  It may be manually specified path, or
        ## a generic name like "konsole".  If it's a generic name we need to find where the icon they
        ## want is really located from the currnetly selected KDE icon theme.
        imageFiles[i] = findIcon(widget, imageFiles[i], str(imageSizeBig) + "x" + str(imageSizeBig))
        if (sleepTime):
            time.sleep(sleepTime)

        ## If the findIcon function returned -1, then use the "unknown" icon
        if (imageFiles[i] == -1):
            imageFiles[i] = findIcon(widget, "unknown", str(imageSizeBig) + "x" + str(imageSizeBig))

        if (sleepTime):
            time.sleep(sleepTime)
        ## Add small image
        images.append(karamba.createImage(widget, borderLeft + centerSmallIcon + (i*iconSeparation), iconFloor, imageFiles[i]))
        ## Add large image
        imagesBig.append(karamba.createImage(widget, borderLeft + (i*iconSeparation), widgetHeight, imageFiles[i]))
        ## Patch Add the Arrows image
        arrows.append(karamba.createImage(widget, borderLeft + (i*iconSeparation)+imageSize/2, iconFloor+imageSize, "arrow.png"))
        karamba.hideImage(widget,arrows[i])

        ## Add clickable area that runs a command -- FIXME, maybe this should be a system command later?
        ##karamba.createClickArea(widget, borderLeft + (i*iconSeparation), (widgetHeight - borderBottom - imageSizeBig - raiseIcons), iconSeparation, imageSizeBig + (raiseIcons/2), commands[i])
        ## Resize the small image to the small image size
        karamba.resizeImageSmooth(widget, images[i], imageSize, imageSize)

    if (sleepTime):
        time.sleep(sleepTime)

    ######################################
    ##  Setup Over Text 		##
    ######################################
    overText=karamba.createText(widget,0,0,widgetWidth,overTextHeight,"")
    karamba.changeTextColor(widget,overText,255,255,255)
    karamba.changeTextSize(widget,overText,overTextSize)
    karamba.changeTextShadow(widget,overText,2)

    #######################################################
    ##  Place widget at the bottom center of the screen  ##
    #######################################################
    if (sleepTime):
        time.sleep(sleepTime)
    if (int(conf['autoPosition']) == 1):
        karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-widgetHeight) )

    ## Update last active time if we're autoHiding (without this autoHiding never starts)
    if (autoHide == 1):
        lastActiveTime = time.time()

    karamba.redrawWidget(widget)
    print "osX Bar " + conf['version'] + " ready [Icon engine by Kroller-0.93b]"



###############################################################################
##  Function:
##    widgetClicked(widget, x, y, button)
##
##  This gets called everytime our widget is clicked.
##  Notes:
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button clicked:
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetClicked(widget, x, y, button):
    """widgetClicked gets called when our widget is clicked"""
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    global ActiveDesktop
    global virtualDesktopRefresh

    if (debug >= 2):
        print "widgetClicked() got called"

    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) > autoHideDelay):
            raiseWidget(widget)

        ## Update last active time
        lastActiveTime = time.time()


    ## LEFT CLICK
    ## Animate icon when an application is launched
    if (button == 1):
        iconNum = (x - borderLeft) / iconSeparation
        ## Patch check for special commands
        if ((iconNum >= 0) and (iconNum < numIcons)):
            XBarClickOverTaskIcon(widget,iconNum,button,x,y)
        if ((iconNum >= 0) and (iconNum < numIcons) and (animateOnClick != "none")):

            ## Bounce Effect
            if (animateOnClick == "bounce"):
                karamba.changeTextColor(widget,overText,255,0,0)
                karamba.changeTextShadow(widget,overText,6)
                ## Going down
                for down in range (1,raiseIcons,4):
                    karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition + down)
                    karamba.redrawWidget(widget)
                    ## Sleep a bit to accomadate the maxFPS setting
                    if (sleepTime):
                        time.sleep(sleepTime)
                ## Going up
                karamba.changeTextColor(widget,overText,0,255,0)
                karamba.changeTextShadow(widget,overText,4)
                for up in range (1,raiseIcons,4):
                    karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition + raiseIcons - up)
                    karamba.redrawWidget(widget)
                    ## Sleep a bit to accomadate the maxFPS setting
                    if (sleepTime):
                        time.sleep(sleepTime)

                karamba.changeTextColor(widget,overText,255,255,255)
                karamba.changeTextShadow(widget,overText,3)
            ## Highlight Effect
            elif (animateOnClick == "highlight"):
                karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.7, 150)
                karamba.redrawWidget(widget)

            ## Gray Effect
            elif (animateOnClick == "gray"):
                karamba.changeImageToGray(widget, imagesBig[iconNum], 200)
                karamba.redrawWidget(widget)

            ## Click Effect
            elif (animateOnClick == "click"):
                ## Go down and to the right (and highlight)
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition + 6, currentYPosition + 6)
                karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.5, 0)
                for i in range(1, conf['maxFPS'], 10):
                    karamba.redrawWidget(widget)
                    time.sleep(sleepTime)
                karamba.removeImageEffects(widget, imagesBig[iconNum])
                ## Go up and to the left
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition)
                karamba.redrawWidget(widget)
        ## Update the virtual desktop selector
        elif ( (conf["virtualDesktopSelector"] == 1) and (x >= xVdBoundaryMin) and (x < xVdBoundaryMax) and (y >= yVdBoundaryMin) and (y <= yVdBoundaryMax) ):
            ## Determine which icon we're over and highlight it
            icon = int((x - xVdBoundaryMin) / conf["virtualDesktopIconX"])
            if (y > yVdBoundaryMax - conf["virtualDesktopIconY"]):
                icon += (conf["numVirtualDesktops"]/2)
            icon += 1
            os.system("dcop kwin KWinInterface setCurrentDesktop " + str(icon))
            karamba.changeImageIntensity(widget, virtualDesktop[icon], 0.7, 150)
            karamba.showImage(widget,virtualDesktop[ActiveDesktop])
            karamba.hideImage(widget,virtualDesktopActive[ActiveDesktop])
            karamba.showImage(widget,virtualDesktopActive[icon])
            virtualDesktopRefresh=0
            ActiveDesktop=icon
            ## karamba.redrawWidget(widget)
    ## MIDDLE CLICK
    ## Show off if middle clicked (this will be replaced with something else in the future!)
    if (button == 2):
        iconNum = (x - borderLeft) / iconSeparation
        ## Patch check for special commands
        if ((iconNum >= 0) and (iconNum < numIcons)):
            XBarClickOverTaskIcon(widget,iconNum,button,x,y)
        ##for i in range (x, (borderLeft + ((numIcons+1) * iconSeparation) - 1), 10):
        ##    widgetMouseMoved(widget, i, (widgetHeight/2), 0)
        ##for i in range ((borderLeft + ((numIcons+1) * iconSeparation)), (borderLeft-iconSeparation), -10):
        ##    widgetMouseMoved(widget, i, (widgetHeight/2), 0)
        ##for i in range ((borderLeft-iconSeparation), x, 10):
        ##    widgetMouseMoved(widget, i, (widgetHeight/2), 0)










###############################################################################
##  Function:
##    widgetUpdated(widget)
##
##  Description:
##    Right now this function is only used for initializing the widget
##
###############################################################################
def widgetUpdated(widget):
    """widgetUpdated gets called at program init and every few seconds"""
    global SysBatteryIndex
    global lastActiveTime
    global widgetCheckForMouse
    global widgetCheckForMouseMax
    global resolutionX
    global widgetWidth
    global resolutionY
    global widgetHeight
    global virtualDesktopRefresh

    ## Update all Icons on async thread
    XBarFakePostUpdateIcon(widget)
    XBarFakePostUpdateText(widget)

    if (debug >= 2):
        print "widgetUpdate got called"
    ## Check to see if we need to autoHide

    XBarListSensorCheck(widget)
    ## For Task Patch
    widgetTaskUpdate(widget)

    ## Update for SysInfo
    XBarEngine.TimeStep()
    ## Update the DesktopSelector Icon
    if (conf["virtualDesktopSelector"] == 1):
        virtualDesktopRefresh=virtualDesktopRefresh+1
        if(virtualDesktopRefresh>5):
            virtualDesktopRefresh=0
            pippo=(widget,)
            thread.start_new_thread(XBarUpdateDesktopSelector,pippo)
        ##XBarUpdateDesktopSelector(widget)

    if(widgetCheckForMouseMax>0):
        x, y = xcursor.position()

        if(x>((resolutionX/2) + (widgetWidth/2)) or x<((resolutionX/2) - (widgetWidth/2))):
            widgetCheckForMouse=widgetCheckForMouse+1
        else:
            if(y<(resolutionY-widgetHeight)):
                widgetCheckForMouse=widgetCheckForMouse+1
            else:
                widgetCheckForMouse=0
        if(widgetCheckForMouseMax<widgetCheckForMouse):
            ## we have to reset the widget
            widgetCheckForMouse=0
            resetWidget(widget)
    ## Other Stuff
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= autoHideDelay):
            resetWidget(widget)
            hideWidget(widget)




###############################################################################
##  Function:
##    widgetMouseMoved(widget, x, y, button)
##
##  This gets called everytime the mouse is moved over our widget
##  Notes
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button being held:
##                    0 = No Mouse Button
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetMouseMoved(widget, x, y, button):
    """widgetMouseMoved gets called everytime the mouse gets moved over our widget"""
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    global isReset
    global numIcons
    global numIconsCfg
    global TheLastIconSelected
    global overTextDelta
    global overTextSpace
    global mouseLastX
    global mouseLastY

    mouseLastX=x
    mouseLastY=y

    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= autoHideDelay):
            raiseWidget(widget)
        ## Update last active time
        lastActiveTime = time.time()

    iconNum = -2
    ## If the mouse is in the target area
    if (x >= xZoomBoundaryMin) and (x < xZoomBoundaryMax) and (y >= yZoomBoundaryMin) and (y <= yZoomBoundaryMax):

        if (debug >= 2):
            print "processing.."

        ## We'll be resizing things, so we're not in a reset state anymore
        isReset = 0

        ## Which icon is the mouse over?
        iconNum = (x - borderLeft) / iconSeparation   ## borderLeft is initial white-space, and iconSeparation pixels between each icon

        ## Unless the current icon comes up as a non-valid number
        if (-2 < iconNum < numIcons+1):

            ## This returns a number between 0 and iconSeparation depending on how far through the image you are
            offset              = (x - borderLeft) % iconSeparation         ## 0 - iconSeparation
            offsetRatio         = (offset/(iconSeparation + 0.0) )          ## Ratio from 0-1 of the current offset
            offsetImageSizeBig  = (offsetRatio * (imageSizeBig/2))          ## 0 - (imageSizeBig/2)
            offsetSizeDiff      = (offsetRatio * sizeDiff)                  ## 0 - sizeDiff
            TheLastIconSelected = iconNum
            ## yRatio lets us do smooth scrolling from the top and bottom
            ## You will see it throughout the script for this reason
            ## If y is in the top half of the icon
            if (y < widgetHeight - borderBottom - raiseIcons - (imageSizeBig/2)):
                yRatio          = ((y + 0.0) / (widgetHeight - borderBottom - raiseIcons - (imageSizeBig/2)))
            else:
                yRatio = 1

            ## If we're positioning the first or last icon, do a special formula
            if (iconNum == -1):
                xShiftLeft = (offset - iconSeparation) * yRatio
                xShiftRight = 0
            elif (iconNum == numIcons):
                xShiftRight = offset * yRatio
                xShiftLeft = 0
            else:
                xShiftLeft = 0
                xShiftRight = 0

            ## Resize the background bar if we're using it
            if (useBackBar == 1):
                karamba.moveImage(widget, backBarLeft, borderLeft - imageSize - (backBarBorder * 2) - ((iconSeparation/2) * yRatio) - (xShiftLeft/2) + (xShiftRight/2), iconFloor - backBarBorder)
                karamba.moveImage(widget, backBarCenter, borderLeft - ((iconSeparation/2) * yRatio) + (xShiftRight/2) - (xShiftLeft/2), iconFloor - backBarBorder)
                karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation)    + ((iconSeparation/2) * yRatio) - (xShiftRight/2) + (xShiftLeft/2), iconFloor - backBarBorder)
                karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight + (iconSeparation * yRatio) + xShiftLeft - xShiftRight + 1, imageSize + (backBarBorder * 2))
            if (useBackBar == 2):
                karamba.moveImage(widget, backBarLeft, 0, widgetHeight)
                karamba.moveImage(widget, backBarCenter,  (xShiftRight/2) + (xShiftLeft/2), iconFloor - backBarBorder)
                karamba.moveImage(widget, backBarRight, 0, widgetHeight)
                karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight +imageSize+ (backBarBorder * 2)+ (iconSeparation * yRatio) + xShiftLeft - xShiftRight + 1, imageSize + (backBarBorder * 2))

            ################################
            ## Size and position iconNum-1
            ################################

            ## Unless we're on the first image, resize the icon to our left
            if iconNum > 0:
                ## Calculate stuff
                ## Size is getting smaller
                size = imageSize + (yRatio * ((sizeDiff/2) - (offsetSizeDiff/2)))
                ## Icon is going left
                ## If we're positioning the last icon, do a special formula
                if (iconNum == numIcons):
                    xpos = ( ((iconNum-1-yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) + (yRatio * ((iconSeparation/2) * offsetRatio)) )
                else:
                    ##        Initial placement                                    center it in it's cell          start half way accross and come back to initial placement
                    xpos = ( ((iconNum-1-yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * ((iconSeparation/2) * offsetRatio)) )
                ## Icon is going down
                ##     center icon                 start from half way up   and go half way down
                ypos = (iconFloor + imageSize - size) - (yRatio * (raiseIcons/2)) + ((yRatio * raiseIcons * offsetRatio)/2)


                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum-1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum-1], xpos, ypos)
                ## Arrows
                karamba.moveImage(widget, arrows[iconNum-1], xpos+size/2, iconFloor+imageSize)

                ## Make sure there are no left-over image effects on the image
                if (currentIconEffect != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum-1])

                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum-1], ((iconNum-1)*iconSeparation)+imageSizeBig, widgetHeight)

                ## Make sure all the previous icons are setup normally
                karamba.moveImage(widget, TaskSeparator, ((numIconsCfg-yRatio)*(iconSeparation)) + borderLeft + xShiftRight -iconSeparation/8, iconFloor)
                for i in range (0, iconNum - 1):
                    karamba.moveImage(widget, images[i], ((i-yRatio)*(iconSeparation)) + borderLeft + centerSmallIcon + xShiftRight, iconFloor)
                    karamba.moveImage(widget, arrows[i], ((i-yRatio)*(iconSeparation)) + borderLeft + xShiftRight + imageSize/2, iconFloor+imageSize)
                    karamba.moveImage(widget, imagesBig[i], ((i-1)*(iconSeparation)) + borderLeft, widgetHeight)

                if (debug >= 2):
                    print "iconNum: ", iconNum-1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos

            ################################
            ## Size and position iconNum
            ################################
            if ( (iconNum > -1) and (iconNum < numIcons) ):
                ## If we're on the left side of the icon
                ## Patch Text
                karamba.changeText(widget,overText,overTextStrings[iconNum])
                overTextDelta=float(len(overTextStrings[iconNum])*overTextSpace)

                if offset < (iconSeparation/2):
                    ## Calculate stuff
                    ## Size is getting larger
                    size = imageSize + (yRatio * ((sizeDiff/2) + offsetSizeDiff))
                    ## Icon is moving right
                    xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * offset) )
                    ## Icon is going up
                    ypos = (iconFloor + imageSize - size) - (yRatio * (raiseIcons/2)) - ((yRatio * raiseIcons * offsetRatio)/2)

                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                    ## Patch Arrows + Text
                    centeredtext=xpos+size/2
                    karamba.moveImage(widget, arrows[iconNum], centeredtext, iconFloor+imageSize)
                    karamba.moveText(widget, overText,centeredtext-overTextDelta,ypos-overTexty)


                ## If we're on the right side of the icon
                else:
                    ## Calculate stuff
                    ## Size is getting smaller
                    size = imageSize + (yRatio * (sizeDiff + (sizeDiff/2) - offsetSizeDiff))
                    ##        initial placement                         center it                       go half way accross and come back offset pixels
                    xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * offset) )
                    ## icon is going down
                    ypos = (iconFloor + imageSize - size) - (yRatio * raiseIcons) + ((yRatio * raiseIcons * offsetRatio)/2)

                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)

                    ## Patch Arrow + Text
                    centeredtext=xpos+size/2
                    karamba.moveImage(widget, arrows[iconNum],centeredtext , iconFloor+imageSize)
                    karamba.moveText(widget, overText,centeredtext-overTextDelta,ypos-overTexty)

                karamba.showText(widget,overText)
                ## Apply current icon effects if any
                if (currentIconEffect != "none"):
                    if (currentIconEffect == "highlight"):
                        karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.2, 0)
                    if (currentIconEffect == "gray"):
                        karamba.changeImageToGray(widget, imagesBig[iconNum], 0)

                ## Move the current small image off the widget
                karamba.moveImage(widget, images[iconNum], (iconNum*iconSeparation) + borderLeft + centerSmallIcon, widgetHeight)

                ## These are used later if the icon is clicked
                currentXPosition = xpos
                currentYPosition = ypos
                currentSize = size

                if (debug >= 2):
                    print "iconNum: ", iconNum, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos





            ################################
            ## Size and position iconNum+1
            ################################

            ## Unless we're on the last image, resize the icon to our right
            if (iconNum < numIcons-1):
                ## Calculate stuff
                size = imageSize + (yRatio * ((offsetSizeDiff/2)))
                ## If we're positioning the first icon, do a special formula
                if (iconNum == -1):
                    xpos = ( ((iconNum+1) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2) * offsetRatio) )
                else:
                    ## icon is going left
                    xpos = ( ((iconNum+1+yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) - (yRatio * (iconSeparation/2) * offsetRatio) )   ## Moves to the left as offset increases
                ## icon is going up
                ypos = (iconFloor + imageSize - size) - ((yRatio * raiseIcons * offsetRatio)/2)


                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum+1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum+1], xpos, ypos)

                ## Patch Arrow
                karamba.moveImage(widget, arrows[iconNum+1], xpos+size/2, iconFloor+imageSize)

                ## Make sure there are no left-over image effects on the image
                if (currentIconEffect != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum+1])

                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum+1], ((iconNum+1)*iconSeparation)+imageSizeBig, widgetHeight)
                karamba.moveImage(widget, TaskSeparator, ((numIconsCfg-1+yRatio)*iconSeparation) + iconSeparation + borderLeft + xShiftLeft -iconSeparation/8, iconFloor)

                ## Make sure all the following icons are setup normally

                for i in range (iconNum+2, numIcons):
                    karamba.moveImage(widget, images[i], ((i-1+yRatio)*iconSeparation) + iconSeparation + borderLeft + centerSmallIcon + xShiftLeft, iconFloor)
                    karamba.moveImage(widget, imagesBig[i], ((i)*iconSeparation) + iconSeparation + borderLeft, widgetHeight)
		    ## Patch Arrow
		    karamba.moveImage(widget, arrows[i], ((i-1+yRatio)*iconSeparation) + iconSeparation + borderLeft + xShiftLeft+imageSize/2, iconFloor+imageSize)

                if (debug >= 2):
                    print "iconNum: ", iconNum+1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
            if (debug >= 2):
                print


            ## Update the screen with the changes we've just made
            karamba.redrawWidget(widget)

# This was a test on highlighting the desktop icon the mouse was currently over
#
#     ################################
#     ##   Update Virtual Desktops  ##
#     ################################
#     elif ( (conf["virtualDesktopSelector"] == 1) and (x >= xVdBoundaryMin) and (x < xVdBoundaryMax) and (y >= yVdBoundaryMin) and (y <= yVdBoundaryMax) ):
#
#         ## Determine which icon we're over and highlight it
#         icon = int((x - xVdBoundaryMin) / conf["virtualDesktopIconX"])
#         if (y > yVdBoundaryMax - conf["virtualDesktopIconY"]):
#             icon += (conf["numVirtualDesktops"]/2)
#         icon += 1
#
#         ## Update the screen with the changes we've just made
#         karamba.redrawWidget(widget)

            

    ## If mouse is to far right or left reset everything to normal
    else:
        if (debug >= 1):
            print "cursor is out of bounds. x: ", x, " y: ", y
        resetWidget(widget)

    
    ## Sleep a bit to accomadate the maxFPS setting
    if (sleepTime):
        time.sleep(sleepTime)











#############################################################################
##  Function:
##    resetWidget(widget)
##
##  Description:
##    Resets the widget to it's normal state, i.e. no icons zoomed
##
#############################################################################
def resetWidget(widget):
	global isReset
	global TaskSeparator
	global numIcons
	global numIconsCfg
	global borderLeft
	global borderRight
	global imageSize
	global backBarBorder
	global iconFloor
	global backBarBorder
	global backBarCenter
	global widgetWidth
	global borderLeft
	global borderRight
	global imageSize

	if(numIcons>numIconsCfg):
		karamba.showImage(widget,TaskSeparator)
	else:
		karamba.hideImage(widget,TaskSeparator)
	karamba.moveImage(widget, TaskSeparator, borderLeft + (numIconsCfg * iconSeparation)-iconSeparation/8, iconFloor)
	karamba.hideText(widget,overText)

	if (isReset == 0):
		## Patch Text
		XBarAnimateReset(widget)
		for i in range (0,numIcons):
			karamba.moveImage(widget, images[i], (i*iconSeparation) + borderLeft + centerSmallIcon, iconFloor)
			karamba.moveImage(widget, imagesBig[i], (i*iconSeparation) + borderLeft, widgetHeight)
			karamba.moveImage(widget, arrows[i], (i*iconSeparation) + borderLeft+imageSize/2, iconFloor+imageSize)
			karamba.removeImageTransformations(widget, imagesBig[i])
	if (useBackBar == 1):
		karamba.moveImage(widget, backBarLeft, borderLeft - imageSize - (backBarBorder * 2), iconFloor - backBarBorder)
		karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
		karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)
		karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
	if (useBackBar == 2):
		karamba.moveImage(widget, backBarLeft, borderLeft - backBarBorder , iconFloor - backBarBorder)
		karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
		karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)
		karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
	karamba.redrawWidget(widget)
	isReset = 1


###############################################################################
## Function:
##   readConfigurationFile ( string $filename )
##
##
## Description:
##   Reads the file $filename, and stores information from the file into
##   the global hash %conf.  This version of the function has been modified
##   specifically to accomadate the needs of Kroller.
##
## Example:
##   readConfigurationFile("/etc/configuration.conf");
##
###############################################################################
def readConfigurationFile(filename):
    """Read a configuration file and store name:value pairs into the global variable conf"""
    global conf
    global commands
    global TaskSleep
    global TaskMax
    global TaskSleepMax
    global imageFiles
    global SpecialCommand
    global SpecialAction
    global overTexty
    global overTextHeight
    global overTextSize
    global TaskListIgnore


    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print "Hint: try putting kroller.conf in ~/.superkaramba/"
        print

    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break

        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("\s+=>\s+", line):               ## If it looks like a valid configuration line

            ## Debugging
            if (int(conf['debug']) >= 6):
                print "Line is: ", line

            ## Extract the name/value pair
            (name, value) = re.split("\s+=>\s+", line, 1)

            ## Add a command
            if (name == "addCommand"):
                commands.append(value)
                TaskSleep.append(0)
            ## Add an icon name
            elif (name == "addIcon"):
                imageFiles.append(value)

            elif (name == "addTaskAlias"):
                commandsAlt.append(value)
            elif (name == "addTextString"):
                overTextStrings.append(value)
            elif (name == "AddSpecialCommand"):
                SpecialCommand.append(value)
            elif (name == "AddSpecialAction"):
                SpecialAction.append(int(value))
            ## The STATIC list of ignored windows
            elif (name == "TaskIgnore"):
                TaskListIgnore.append(value)
            ## EMails configurations
            elif (name == "EMailAddServer"):
                SysEMailServers.append(value)
            elif (name == "EMailAddUser"):
                SysEMailUsers.append(value)
            elif (name == "EMailAddPasswd"):
                SysEMailPasswords.append(value)
            elif (name == "addSensorType"):
                XBarEngine.SysSensorListType.append(value)
            elif (name == "addSensorIndex"):
                XBarEngine.SysSensorListIndex.append(value)
            elif (name == "addSensorCmd"):
                XBarEngine.SysSensorListCmd.append(value)
            elif (name == "addSensorRefresh"):
                XBarEngine.SysSensorListMax.append(value)
            elif (name == "addSensorAliasCmd"):
                XBarSensor.SysSensorsAliasCmd.append(value)
            elif (name == "addSensorAliasIcon"):
                XBarSensor.SysSensorsAliasIcon.append(value)
            elif (name == "addSensorAlias"):
                XBarSensor.SysSensorsAlias.append(value)
            ## Must be a generic configuration parameter, store it in conf
            else:
                if (re.match("^\d+$", value)):
                    conf[name] = int(value)
                else:
                    conf[name] = value

    ## Close the coniguration file
    file.close()


    ## DEBUGGING:
    if (int(conf['debug']) >= 3):
        print "Configuration: "
        for key in conf.keys():
            print key, "\t=>\t", conf[key]










###############################################################################
## Function:    
##   getKDEConfigValue ( string $filename, string $section, string $option )
## 
##
## Description: 
##   Reads the file $filename, searches for the section $section (each section
##   is delimeted by  "[section name]" in KDE config files, finds the option
##   $option and returns the value.
## 
## Example:
##   num = getKDEConfigValue("/root/.kde/share/kdeglobals", "Desktops", "Number");
##
###############################################################################
def getKDEConfigValue(filename, section, option):
    """Read a KDE config file and return the value for option under the specified section"""
    
    ## FIXME: Should this function be replaced with calls to 
    ## pythons ConfigParser?

    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print
    
    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break
        
        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("^\[" + section + "\]", line):           ## If we're entering the right section
            while 1:
                line = file.readline()
                if len(line) == 0:                            ## Break this while loop if we're at the end of the file
                    break
                line = line.strip()
                if re.search("^" + option + "=", line):
                    (name, value) = re.split("=", line, 1)
                    file.close
                    return(str(value))

    ## Close the coniguration file and return an error if we didn't find what we were looking for
    file.close()
    return(-1)
    










###############################################################################
## Function:    
##   findIcon ( widget, string $iconName, string $size )
##
## 
## Description:
##   Trys really hard to find an icon based on the text given in $icon.
##   If $icon is an absolute path to an icon $icon will be returned 
##   unchanged.  Otherwise it will look for an icon in various KDE icon
##   directories that matches the name given.
##   If a $size is specified it will try to find one of that size in the
##   KDE dirs.
##  
## Example:
##   iconPath = findIcon("konsole", "96x96");
##
###############################################################################
def findIcon(widget, icon, size):
    """Try to find an icon"""
    global kdeIconPaths
    
    ## Try to build the kdeIconPaths array only once to save CPU
    if (not kdeIconPaths):
        
        ## Find the KDE icon theme
        kdeIconTheme = getKDEConfigValue(os.getenv("HOME") + "/.kde/share/config/kdeglobals", "Icons", "Theme")
        if ( (kdeIconTheme == -1) or (kdeIconTheme == "") or (re.match("^\d+$", kdeIconTheme)) ):
            print "KDE has no icon theme configured.  Using default.kde"
            kdeIconTheme = "default.kde"
        print "Current KDE Icon Theme: " + kdeIconTheme

        ## Get the paths that KDE is using for icons
        fp = os.popen("kde-config --path icon")
        output = []
        if (fp):
            output = fp.read()
            ## Strip newlines
            output = output.replace('\n', '')
            output = output.split(":")
        
        ## KDEDIR should probably be removed from future kroller versions
        ## and rely solely on the kde-config command, since KDEDIR var
        ## is being depricated.
        kdedir = os.getenv("KDEDIR")
        if (not kdedir):
            fp = os.popen("kde-config --prefix")
            if (fp):
                tmp = fp.read()
                tmp = tmp.replace('\n', '')
                kdedir = tmp
                del tmp
        if (not kdedir):
            kdedir = "/opt/kde"
        print "Detected KDEDIR: " + str(kdedir)
        if (not os.path.exists(kdedir)):
            print "ERROR - Could not locate the KDE installation directory!"
        
        
        ## Now look in KDE's icon paths
        kdeIconPaths = []
        
        ## These few entries *should* find everyones icons
        for entry in output:
            kdeIconPaths.append(entry + kdeIconTheme)
        
        ## Make sure to check the kroller theme dir for icons
        kdeIconPaths.append(karamba.getThemePath(widget))
        kdeIconPaths.append(karamba.getThemePath(widget) + "/icons")
        
        ## These should have already been added from the kde-config command, but just in case
        kdeIconPaths.append(os.getenv("HOME") + "/.kde/share/icons/" + kdeIconTheme)
        kdeIconPaths.append(kdedir + "/share/icons/" + kdeIconTheme)
        
        ## As a fallback check the default.kde and hicolor dirs
        for entry in output:
            kdeIconPaths.append(entry + "default.kde")
        for entry in output:
            kdeIconPaths.append(entry + "hicolor")
        
        ## Add a few more search paths just for paranoia's sake
        kdeIconPaths.append("/usr/share/icons" + kdeIconTheme)
        kdeIconPaths.append("/usr/share/icons")
        kdeIconPaths.append(kdedir + "/share/icons/default.kde")
        kdeIconPaths.append(kdedir + "/share/icons/hicolor")
        
        if (debug >= 1):
            print "Icon Search Paths:"
            for path in kdeIconPaths:
                print "    " + str(path)

    
    sizes = []
    if (size):
        sizes.append(size)
    sizes.append("128x128")
    sizes.append("96x96")
    sizes.append("64x64")
    sizes.append("48x48")
    sizes.append("32x32")
    sizes.append("22x22")
    sizes.append("16x16")
    
    types = []
    types.append("apps")
    types.append("devices")
    types.append("filesystems")
    types.append("mimetypes")

    
    ## If an absolute path was defined use that
    if (os.path.exists(icon)):
        if (debug >= 1):
            print "Found icon for: " + icon + " at path: " + icon
        return(icon)
    
    ## Look for the icon in all of the search paths discovered above
    for path in kdeIconPaths:
        if (os.path.exists(path + "/" + icon + ".png")):
            if (debug >= 1):
                print "Found icon for: " + icon + " at path: " + path + "/" + icon + ".png"
            return(path + "/" + icon + ".png")
        if (os.path.exists(path + "/" + icon)):
            if (debug >= 1):
                print "Found icon for: " + icon + " at path: " + path + "/" + icon
            return(path + "/" + icon)
        for size in sizes:
            for type in types:
                if (os.path.exists(path + "/" + size + "/" + type + "/" + icon + ".png")):
                    if (debug >= 1):
                        print "Found icon for: " + icon + " size: " + size + " at path: " + path + "/" + size + "/" + type + "/" + icon + ".png"
                    return(path + "/" + size + "/" + type + "/" + icon + ".png")
                if (os.path.exists(path + "/" + size + "/" + type + "/" + icon)):
                    if (debug >= 1):
                        print "Found icon for: " + icon + " size: " + size + " at path: " + path + "/" + size + "/" + type + "/" + icon
                    return(path + "/" + size + "/" + type + "/" + icon)
    
    ## Return -1 if we couldn't find the icon
    print "ERROR - Coundn't find an icon for \"" + icon + "\""
    return(-1)











###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description:
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
##
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
    """Detect Current XFree86 Resolution"""
    global resolutionX
    global resolutionY
    
    havexwininfo = os.system("which xwininfo >/dev/null")
    
    if (havexwininfo == 0):
        pass
    else:
        resolutionX = 1024
        resolutionY = 768
        print "\nCan't find xwininfo in your path!  Guessing resolution is 1024x768."
    
    fp = os.popen("xwininfo -root -stats")
    output = fp.read()
    output = output.splitlines()
    i = 0
    for x in output:
        param = x.split()
        if (len(param) > 1):
            if param[0].find("Width:") != -1:
                resolutionX = int(param[1])
            if param[0].find("Height:") != -1:
                resolutionY = int(param[1])
    
    print "Detected resolution: " + str(resolutionX) + "x" + str(resolutionY)









###############################################################################
## Function:
##   hideWidget ( widget )
## 
##
## Description: 
##   Hides kroller
## 
###############################################################################
def hideWidget(widget):
    
    karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-borderBottom) )










###############################################################################
## Function:    
##   raiseWidget ( widget )
## 
##
## Description: 
##   Shows kroller
## 
###############################################################################
def raiseWidget(widget):
    karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-widgetHeight) )

# This will be printed when the widget loads.
print "osX Bar version " + conf["version"] + " starting up."
print "you may check for updates on:"
print "http://www.kde-look.org"
print "http://www.xiaprojects.com"

